<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 15/07/14
 * Time: 13:41
 */

namespace Service;


use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Util\Messages;
use Util\Utilities;

use Service\Allocation;
use Service\Resource;
use Service\Task;
use Service\Attachment;
use \Zeedhi\DevOps\Model\Projectmanagement\Repositories;
use \Zeedhi\DevOps\Model\Projectmanagement\Entities;

class ApiProjectManagement {

    public static $PROJECT_FIELDS       = array("NAME", "MANAGER", "SPONSOR", "START");
    public static $TASK_FIELDS          = array("PROJECT", "NAME");
    public static $APPROPRIATION_FIELDS = array("TASK_ID", "RESOURCE", "DESCRIPTION", "PERCENT_COMPLETED", "BEGIN_HOUR", "END_HOUR");
    public static $PROFILE_FIELDS       = array( "NAME");

    /** @var \Doctrine\ORM\EntityManager $entityManager */
    private $entityManager;
    /** @var \Service\Resource $allocationService */
    private $allocationService;
    /** @var \Service\Resource $resourceService */
    private $resourceService;
    /** @var \Service\Task $taskService */
    private $taskService;
    /** @var \Service\Structure $structureService */
    private $structureService;
    /** @var \Service\Project $projectService */
    private $projectService;
    /** @var Acl $aclService */
    private $aclService;
    /** @var \Service\Attachment $attachmentService */
    private $attachmentService;
    /** @var  \Factory\Comments $commentsFactory */
    private $commentsFactory;

    /** @var Utilities $utilities */
    private $utilities;

    /** @var Repositories\Resource $utilities */
    private $resourceRepository;

    /** @var Repositories\Task $utilities */
    private $taskRepository;

    public function __construct(EntityManager $entityManager, Resource $resourceService, Allocation $allocationService, Task $taskService,
        Structure $structureService, Project $projectService, Acl $aclService, Attachment $attachmentService, \Factory\Comments $commentsFactory, Utilities $utilities){
        $this->entityManager = $entityManager;
        $this->resourceService = $resourceService;
        $this->taskService = $taskService;
        $this->allocationService = $allocationService;
        $this->structureService = $structureService;
        $this->projectService = $projectService;
        $this->aclService = $aclService;
        $this->attachmentService = $attachmentService;
        $this->commentsFactory = $commentsFactory;
        $this->utilities = $utilities;

        $this->taskRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Task');
        $this->resourceRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Resource');
    }

    private function validateObject($object, $keysToValidate){
        foreach ($keysToValidate as $key) {
            if(empty($object[$key])){
                throw new \Exception("Required field: {$key}.");
            }
        }
    }

    public function buildProject($project){
        $this->validateObject($project, self::$PROJECT_FIELDS);
        $manager = $this->resourceService->getResourceIdByEmail($project['MANAGER']);
        if(empty($manager)){
            throw new \Exception("Manager not found.");
        }
        $group = $this->structureService->getFirstStructureId($manager);
        if(empty($group)){
            throw new \Exception("Manager group not founded.");
        }
        return array(
            "NAME"    => $project['NAME'],
            "MANAGER" => $manager,
            "SPONSOR" => $this->resourceService->getResourceIdByCNPJ($project['SPONSOR']),
            "GROUP"   => $group["STRUCTURE"],
            "START"   => $project['START'],
            "END"     => empty($project['END']) ? $project['START'] : $project['END'],
        );
    }

    public function buildTask($task){
        $this->validateObject($task, self::$TASK_FIELDS);
        $project = $this->projectService->getProjectById($task['PROJECT']);
        if(empty($project)){
            throw new \Exception("Project {$task['PROJECT]']} not found.");
        }
        $task = array(
            "PROJECT"     => $task['PROJECT'],
            "NAME"        => $task['NAME'],
            "DESCRIPTION" => empty($task['DESCRIPTION']) ? null : $task['DESCRIPTION'],
            "TOTAL_TIME"  => empty($task['TOTAL_TIME']) ? '8H' : $task['TOTAL_TIME']
        );
        $task["START"] = $project->getBeginDate()->format('Y-m-d');
        $task["END"] = new \DateTime($task['START']);
        $task["END"]->add(new \DateInterval("PT{$task['TOTAL_TIME']}"));
        $task["END"] = $task["END"]->format('Y-m-d');
        return $task;
    }

    public function buildAppropriation($appropriation){
        $this->validateObject($appropriation, self::$APPROPRIATION_FIELDS);
        $resourceId = $this->resourceService->getResourceIdByEmail($appropriation['RESOURCE']);
        $allocation = $this->allocationService->getAllocationByUserAndTask($resourceId, $appropriation["TASK_ID"]);
        if(empty($resourceId) || empty($allocation)){
            throw new \Exception("Resource or task not founded.");
        }
        $appropriation = array(
            "ALLOCATION"        => $allocation->getId(),
            "DESCRIPTION"       => $appropriation['DESCRIPTION'],
            "PERCENT_COMPLETED" => $appropriation['PERCENT_COMPLETED'],
            "BEGIN_HOUR"        => $appropriation['BEGIN_HOUR'],
            "END_HOUR"          => $appropriation['END_HOUR']
        );
        $beginHour = new \DateTime($appropriation['BEGIN_HOUR']);
        $endHour = new \DateTime($appropriation['END_HOUR']);
        $appropriation['TOTAL'] = $endHour->getTimestamp() - $beginHour->getTimestamp();
        return $appropriation;
    }
    public function findProfileIdByExternalId($profileExternalId){
        if(empty($profileExternalId)){
            $profileId = null;
        }else{
            $profileEntity = $this->aclService->findProfileByExternalId($profileExternalId);
            if(empty($profileEntity)){
                $profileId = null;
            }else{
                $profileId = $profileEntity->getId();
            }
        }
        return $profileId;
    }
    public function findResourceIdByExternalId($resourceExternalId){
        if(empty($resourceExternalId)){
            $resourceId = null;
        }else{
            $resourceEntity = $this->resourceService->getResourceByExternalId($resourceExternalId);
            if(empty($resourceEntity)){
                $resourceId = null;
            }else{
                $resourceId = $resourceEntity->getId();
            }
        }
        return $resourceId;
    }
    public function buildProfile($profile){
        $this->validateObject($profile, self::$PROFILE_FIELDS);
        if(empty($profile['EXTERNAL_ID'])){
            $profile['EXTERNAL_ID'] = null;
        }else{
            $profileEntity = $this->aclService->findProfileByExternalId($profile['EXTERNAL_ID']);
            if(empty($profileEntity)){
                $profile['ID'] = null;
            }else{
                $profile['ID'] = $profileEntity->getId();
            }
        }
        $profile = array(
            "ID"               => $profile['ID'],
            "STRUCTURE"        => empty($profile['STRUCTURE']) ? 0 : $profile['STRUCTURE'],
            "CATEGORY_CONTROL" => empty($profile['CATEGORY_CONTROL']) ? 0 : $profile['CATEGORY_CONTROL'],
            "CONTROL"          => empty($profile['CONTROL']) ? null : $profile['CONTROL'],
            "NAME"             => $profile['NAME']
        );
        return $profile;
    }

    public function findTaskByUser($resource, $beginDate, $endDate){
        $resourceId = $this->resourceService->getResourceIdByEmail($resource);
        return $this->taskService->getTasksByUserPerPeriod($resourceId, $beginDate, $endDate);
    }
    public function deleteResource($id){
        return $this->resourceService->deleteResource($id);
    }

    public function findResourceIdByEmail($resource){
        return $this->resourceService->getResourceIdByEmail($resource);
    }
    public function findAllManager(){
        return $this->resourceService->findAllManager();
    }

    public function createIssueByProductCall($name, $classification, $description, $board, $due_date, $sponsor = null,
                                             $reporterCdoperador = null)
    {
        $issueStatus = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'TaskStatus')
            ->findOneBy(array('control' => Task::TASK_STATUS_CONTROL_BACKLOG))->getId();
        $reporterId = $this->utilities->getCurrentUserId();
        $dueDate = $due_date;
        $priority = null;
        $percentCompleted = 0;
        $followers = array();
        $labels = null;

        try {
            $reporterId = $this->saveContactBySponsor($reporterCdoperador);
            Utilities::getEnvironment()->setCurrentUserId($reporterId);
            $issueId = $this->taskService->saveNewIssue($name, $classification, $issueStatus, $reporterId, $dueDate, $priority, $percentCompleted, $description,
                $board, $followers, $labels, $sponsor);

            return $this->taskService->findIssue($issueId)[0];
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function updateIssueByProductCall($id, $name, $classification, $description, $board, $due_date) {
        $issue              = $this->taskService->findIssue($id)[0];
        $issueStatus        = $issue['ISSUE_STATUS'];
        $reporter           = $issue['REPORTER'];
        $dueDate            = $due_date;
        $priority           = $issue['PRIORITY'];
        $percentCompleted   = $issue['PERCENT_COMPLETED'];
        $followers          = $issue['FOLLOWERS'];
        $sponsor            = $issue['SPONSOR_ID'];
        $labels             = $issue['LABELS'];

        
        try{
            $issueId = $this->taskService->saveIssue($id, $name, $classification, $issueStatus, $reporter, $dueDate, $priority, $percentCompleted,
                $description, $followers, null, $board, $labels, $sponsor);

            return $this->taskService->findIssue($issueId['ID'])[0];
        } catch(\Exception $e){
            return null;
        }
    }

    public function removeIssueById($issueId) {
        try{
            $this->taskService->deleteIssues($issueId);
            return true;
        } catch(\Exception $e) {
            return false;
        }

    }


    public function saveContactBySponsor($externalCode)
    {
        if ($externalCode) {
            /** @var  Entities\Resource $reporter */
            $reporter = $this->resourceRepository->findOneBy(array('externalCode' => $externalCode));
            Utilities::getEnvironment()->setCurrentCdoperador($externalCode);
            if ($reporter) {
                $reporterId = $reporter->getId();
            } else {
                $contactName = $this->resourceRepository->findNameOperador($externalCode);
                if (!empty($contactName)) {
                    $contactName = $contactName[0]["NMOPERADOR"];
                }else{
                    $contactName = 'Operador sem nome';
                }
                $sponsorId = $this->resourceRepository->findSponsorIdByCdoperador($externalCode);
                if (empty($contactName)) {
                    throw new \Exception('Cliente não relacionado ao sistema de gestão de projetos. Porfavor entrar em contado com o administrador do sistema');
                }else{
                    $sponsorId = $sponsorId[0]['ID'];
                }

                $reporterId = $this->resourceService->createContactByProductCall($sponsorId, $contactName,$externalCode)->getId();
            }
        }
        if(empty($reporterId)){
            throw new \Exception('Reporter not found');
        }else{
            return $reporterId;

        }
    }

    private function objectsByID($array) {
        $newArray = array();
        foreach($array as $element) {
            $newArray[$element['ID']] = $element;
        }
        return $newArray;
    }
    public function setStatusIssue(Entities\Task $issue){
        /** @var Entities\Comments $lastComment */
        $lastComment = InstanceProvider::getRepository()->getComments()->findOneBy(array('task' => $issue), array('createdAt' => 'DESC'));

        if($lastComment && $lastComment->getCommentType()->getControl() == Entities\CommentType::TYPE_QUESTION){
            /** @var Entities\TaskStatus $newTaskStatus */
            $newTaskStatus = InstanceProvider::getRepository()->getTaskStatus()->findOneBy(array('control' => Appropriation::CONTROL_BACKLOG,
                'usedInIssue' => Appropriation::USED_IN_ISSUE_YES,
                'organization' => $issue->getOrganization()));
            if(!empty($newTaskStatus)){
                $issue->setTaskStatus($newTaskStatus);
            }
            $taskRelationships = InstanceProvider::getRepository()->getTaskRelationship()->findBy(array('task' => $issue, 'type' => TaskPrecursor::RELATIONSHIP_ISSUE_TYPE));
            InstanceProvider::getService()->getAppropriation()->setStatusIssues(false, $taskRelationships,false, false);
        }
    }

    public function saveCommentOnIssuesByProductCallComment($issueId, $description,$cdoperador = null) {
        $connection  = $this->entityManager->getConnection();
        $sendEmail = false;
        try {
            $this->entityManager->beginTransaction();
            /** @var Entities\Task $issue */
            $issue = $this->taskRepository->find($issueId);
            $reporterId = $this->saveContactBySponsor($cdoperador);
            Utilities::getEnvironment()->setCurrentUserId($reporterId);
            $this->setStatusIssue($issue);
            $commentType = InstanceProvider::getRepository()->getCommentType()->findOneBy(array('control'=>Entities\CommentType::TYPE_COMMON));
            $comment = $this->commentsFactory->createComments($issue, $description,$commentType);
            $issue->setCurrentTaskStatusIn(Utilities::getEnvironment()->getCurrentTime());
            $this->entityManager->persist($issue);
            $this->entityManager->persist($comment);
            $this->entityManager->flush();
            InstanceProvider::getEnvironment()->setCurrentUserId($reporterId);

            $connection->commit();
            $sendEmail = true;
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        if($sendEmail){
            InstanceProvider::getService()->getComment()->sendsMailAdvisingAboutNewIssueComment($issueId, $description);
        }
    }

    public function linkDownloadBudgetSm($issueId) {
        return \Util\Instance\Service::getIssue()->linkDownloadBudgetSm($issueId);
    }

    public function saveReopenIssueByProductCall($issueId, $description,$cdoperador = null) {
        $connection  = $this->entityManager->getConnection();
        try {
            $this->entityManager->beginTransaction();
            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task $issue */
            $issue = $this->taskRepository->find($issueId);
            $reporterId = $this->saveContactBySponsor($cdoperador);
            Utilities::getEnvironment()->setCurrentUserId($reporterId);
            $commentType = InstanceProvider::getRepository()->getCommentType()->findOneBy(array('control'=>Entities\CommentType::TYPE_REFUSED));
            $comment = $this->commentsFactory->createComments($issue, $description,$commentType);
            $issue->setTaskStatus(Utilities::getTaskStatusBacklog());
            $issue->setUsedInFraming('N');
            $issue->setCurrentTaskStatusIn(Utilities::getEnvironment()->getCurrentTime());
            $this->entityManager->persist($issue);
            $this->entityManager->persist($comment);
            $this->entityManager->flush();

            $connection->commit();
            InstanceProvider::getService()->getTask()->sendMailOnReopenIssue($issueId);
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function saveRefuseIssueByProductCall($issueId, $description,$cdoperador = null,$document = null) {
        try {
            $reporterId = $this->saveContactBySponsor($cdoperador);
            Utilities::getEnvironment()->setCurrentUserId($reporterId);
            $commentType = InstanceProvider::getRepository()->getCommentType()->findOneBy(array('control'=>Entities\CommentType::TYPE_REFUSED));
            InstanceProvider::getService()->getIssue()->clientRefuse($issueId,$commentType->getId(),$description,$document,false);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function saveApproveIssueByProductCall($issueId, $description,$cdoperador = null,$document = null) {
        try {
            $reporterId = $this->saveContactBySponsor($cdoperador);
            Utilities::getEnvironment()->setCurrentUserId($reporterId);
            $commentType = InstanceProvider::getRepository()->getCommentType()->findOneBy(array('control'=>Entities\CommentType::TYPE_ACCEPT));
            InstanceProvider::getService()->getIssue()->clientAccept($issueId,$commentType->getId(),$description,$document,false);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function saveGiveUpImplementationIssue($issueId, $description,$cdoperador = null,$document = null) {
        try {
            $reporterId = $this->saveContactBySponsor($cdoperador);
            Utilities::getEnvironment()->setCurrentUserId($reporterId);
            $commentType = InstanceProvider::getRepository()->getCommentType()->findOneBy(array('control'=>Entities\CommentType::TYPE_ACCEPT));
            InstanceProvider::getService()->getIssue()->saveGiveUpImplementationIssue($issueId,$commentType->getId(),$description,$document,false);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function findCommentsOnIssueForProductCall($issueId) {
        return $this->taskService->findCommentsOnIssueExceptCommentedBy($issueId);
    }

    public function uploadIssueAttachments($issueId, $documents,$cdoperador, $commentId = null) {

        $reporterId = $this->saveContactBySponsor($cdoperador);
        Utilities::getEnvironment()->setCurrentUserId($reporterId);
        $attachmentsUploaded  = $this->attachmentService->addDocumentsTask($issueId, $documents, $commentId);
        $attachmentsUploadedIds = array();
        foreach($attachmentsUploaded as $attachment) {
            $attachmentsUploadedIds[] = $attachment->getId();
        }
        return $attachmentsUploadedIds;
    }

    public function getIssueById($id) {
        return $this->attachmentService->getIssueById($id);
    }

    public function getIssueAttachmentsDataSet($issueId) {
        return $this->attachmentService->getIssueAttachmentsExceptUploadedBy($issueId);
    }
}